OBJDIR	= obj

# Cross-compiler toolchain
CC	= $(GCCPREFIX)gcc -pipe
CXX	= $(GCCPREFIX)c++ -pipe
AS	= $(GCCPREFIX)as
AR	= $(GCCPREFIX)ar
LD	= $(GCCPREFIX)ld
OBJCOPY	= $(GCCPREFIX)objcopy
OBJDUMP	= $(GCCPREFIX)objdump
NM	= $(GCCPREFIX)nm
STRIP	= $(GCCPREFIX)strip

# Native commands
NCC	= gcc $(CC_VER) -pipe
TAR	= tar
PERL	= perl

# Compiler flags
# Note that -O2 is required for the boot loader to fit within 512 bytes;
# -fno-builtin is required to avoid refs to undefined functions in the kernel.
CFLAGS	:= $(CFLAGS) $(DEFS) $(LABDEFS) -Os -fno-builtin -I. -MD -Wall -Wno-format -Werror -ggdb

# Different flags for kernel and user programs
KERN_CFLAGS = $(CFLAGS) -DWEENYOS_KERNEL
USER_CFLAGS = $(CFLAGS) -DWEENYOS_USER


# This magic automatically generates makefile dependencies
# for header files included from C source files we compile,
# and keeps those dependencies up-to-date every time we recompile.
# See 'mergedep.pl' for more information.
$(OBJDIR)/.deps: $(wildcard $(OBJDIR)/*.d)
	@mkdir -p $(@D)
	@$(PERL) mergedep.pl $@ $^

-include $(OBJDIR)/.deps
-include conf/date.mk


ifndef GCCPREFIX
GCCPREFIX := $(shell if i386-jos-elf-objdump -i 2>&1 | grep '^elf32-i386$$' >/dev/null 2>&1; \
	then echo 'i386-jos-elf-'; \
	elif objdump -i 2>&1 | grep 'elf32-i386' >/dev/null 2>&1; \
	then echo ''; \
	else echo "***" 1>&2; \
	echo "*** Error: Couldn't find an i386-*-elf version of GCC/binutils." 1>&2; \
	echo "*** Is the directory with i386-jos-elf-gcc in your PATH?" 1>&2; \
	echo "*** If your i386-*-elf toolchain is installed with a command" 1>&2; \
	echo "*** prefix other than 'i386-jos-elf-', set your GCCPREFIX" 1>&2; \
	echo "*** environment variable to that prefix and run 'make' again." 1>&2; \
	echo "*** To turn off this error, run 'gmake GCCPREFIX= ...'." 1>&2; \
	echo "***" 1>&2; exit 1; fi)
endif


# The mkbootdisk program
mkbootdisk: mkbootdisk.c
	@echo + hostcc mkbootdisk.c
	$(V)gcc mkbootdisk.c -o mkbootdisk


# For deleting the build
clean:
	@echo + clean
	$(V)rm -rf $(OBJDIR) *.img mkbootdisk bochs.log

realclean: clean
	$(V)rm -rf $(DISTDIR)-$(USER).tar.gz $(DISTDIR)-$(USER)

distclean: realclean


# Distribute weensyos
DISTDIR = weensyos1

distdir:
	@/bin/rm -rf $(DISTDIR)
	mkdir $(DISTDIR)
	for i in .bochsrc COPYRIGHT GNUmakefile bootstart.S elf.h mergedep.pl mkbootdisk.c mmu.h passwdos-boot.c passwdos-c.c passwdos-p.c passwdos-yield.S passwdos.h print111os-boot.c types.h x86.h conf conf/rules.mk; do \
	[ -d $$i ] && mkdir $(DISTDIR)/$$i; \
	[ -f $$i ] && ln $$i $(DISTDIR)/$$i; done
	echo >$(DISTDIR)/conf/date.mk "PACKAGEDATE="`date`

dist: distdir
	$(TAR) czf $(DISTDIR).tar.gz $(DISTDIR)
	/bin/rm -rf $(DISTDIR)

tarball: realclean
	@echo + mk $(DISTDIR)-$(USER).tar.gz
	$(V)mkdir $(DISTDIR)-$(USER)
	$(V)tar cf - `ls -a | grep -v '^\.*$$' | grep -v '^$(DISTDIR)*'` | (cd $(DISTDIR)-$(USER) && tar xf -)
	$(V)/bin/rm -rf `find $(DISTDIR)-$(USER) -name CVS -print`
	$(V)/bin/rm -rf `find $(DISTDIR)-$(USER) -name .svn -print`
	$(V)date > $(DISTDIR)-$(USER)/tarballstamp
	$(V)tar cf $(DISTDIR)-$(USER).tar $(DISTDIR)-$(USER)
	$(V)gzip $(DISTDIR)-$(USER).tar
	$(V)/bin/rm -rf $(DISTDIR)-$(USER)

handin: tarball
	@echo + handin
	$(V)uuencode $(DISTDIR)-$(USER).tar.gz < $(DISTDIR)-$(USER).tar.gz | mail -s $(DISTDIR)-$(USER).tar.gz osp-s05-handin@pdos.lcs.mit.edu

run-%: %.img
	@echo + bochs $<
	$(V)/bin/mv .bochsrc .bochsrc~
	$(V)sed 's/path="[^"]*"/path="$<"/' <.bochsrc~ >.bochsrc
	$(V)/bin/rm .bochsrc~
	$(V)bochs -q


# Always build something
always:
	@:

# These are phony targets
.PHONY: all always clean realclean distclean dist distdir tarball handin

# Eliminate default suffix rules
.SUFFIXES:

# Delete target files if there is an error (or make is interrupted)
.DELETE_ON_ERROR:

# no intermediate .o files should be deleted
.PRECIOUS: %.o $(OBJDIR)/%.o $(OBJDIR)/%-bootsector
