#include "x86.h"
#include "passwdos.h"

/*****************************************************************************
 * passwdos-c
 *
 *   This is the passwdos's "password checker" process.
 *   It reads the password, one character at a time, from the "pipe
 *   buffer" into a local variable (buf[], in the start() function).
 *   Then it compares the password it read to "good_password".
 *   If the input password equals good_password, then it prints "Y" to
 *   the console.  If it doesn't, it prints "N".
 *
 *****************************************************************************/

char good_password[] = "111";

char
getch(void)
{
	char result;

	// Wait until the pipe buffer is nonempty.
	while (*PIPEBUF == 0)
		_yield(PROCESSID_C, PROCESSID_P);

	// Read the character, then clear the buffer and return.
	result = *PIPEBUF;
	*PIPEBUF = 0;
	return result;
}

int
strcmp(const char *a, const char *b)
{
	// Returns 0 if and only if the two strings, 'a' and 'b',
	// contain the same characters in the same order.
	while (*a && *b && *a == *b)
		a++, b++;
	return (int) ((unsigned char) *a) - (int) ((unsigned char) *b);
}

void
write_char(char c)
{
	// Write a character 'c' to the upper-left corner of the console,
	// then erase everything else.
	// For explanation, see 'print111os-boot.c'.
	
	int i;
	char *console = (char *) 0xB8000;
	
	console[0] = c;
	console[1] = 0x07;

	for (i = 1; i < 80 * 25; i++) {
		console[i*2] = ' ';
		console[i*2 + 1] = 0x07;
	}

	outb(0x3D4, 14);
	outb(0x3D5, 1 / 256);		// That is, 0
	outb(0x3D4, 15);
	outb(0x3D5, 1);
}

void
start(void)
{
	char buf[101];
	char output;
	int pos;

	// Read at most 100 characters.
	for (pos = 0; pos < 100; pos++) {
		char c = getch();
		// A newline character ('\n') represents the end of
		// the password.
		if (c == '\n')
			break;
		else
			buf[pos] = c;
	}
	// Make sure the string is null-terminated.
	buf[pos] = 0;

	// Now we have the user's password.
	// Compare it to 'good_password'.
	if (strcmp(buf, good_password) == 0)
		output = 'Y';
	else
		output = 'N';

	// Output the result.
	write_char(output);

	// We're done -- do nothing for the rest of time!
	while (1)
		_yield(PROCESSID_C, PROCESSID_P);
}
